# BluetoothLibraryPatcher
# hexpatch script
# by 3arthur6

bb=/tmp/bluetoothlibpatcher/bin/busybox

ui_print() {
  echo -n -e "ui_print $1\n" > /proc/self/fd/$OUTFD
  echo -n -e "ui_print\n" > /proc/self/fd/$OUTFD
}

umountsysven() {
  ui_print "- Unmounting system & vendor"
  $bb umount /system || $bb umount /system_root
  $bb umount /vendor
}

abort() {
  umountsysven
  $bb rm -rf /tmp/bluetoothlibpatcher
  ui_print "- Aborting"
  ui_print " "
  ui_print " "
  exit 1
}

exit_patch() {
  umountsysven
  $bb rm -rf /tmp/bluetoothlibpatcher
  ui_print "- Exiting"
  ui_print " "
  ui_print " "
  exit 0
}

clean() {
  umountsysven
  $bb rm -rf /tmp/bluetoothlibpatcher
  ui_print "- Done"
  ui_print " "
  ui_print " "
  exit 0
}

check() {
  samsung=`$bb grep -Eqw "androidboot.odin_download|androidboot.warranty_bit|sec_debug" /proc/cmdline && echo 'true' || echo 'false'`
  sys=`$bb dirname $($bb find /system* -mindepth 2 -maxdepth 3 -path "*system/build.prop"|head -1)`
  aosp=`$bb grep -qw ro.build.type=user $sys/build.prop && echo 'false' || echo 'true'`
  sdk=`$bb grep ro.build.version.sdk= $sys/build.prop|$bb cut  -d "=" -f2`

  if ! $samsung ; then
    ui_print "- Only for Samsung devices!"
    abort
  elif $aosp ; then
    ui_print "- Only for Samsung stock based roms!"
    ui_print "- Not relevant for aosp roms"
    abort
  elif [[ $sdk -lt 24 ]] ; then
    ui_print "- Only for Android 7.0 (Nougat) and above"
    abort
  fi
}

search() {
  bootloader=`$bb grep -o androidboot.bootloader=.* /proc/cmdline|$bb cut -d ' ' -f1|$bb cut -d '=' -f2`
  qcom=`$bb grep -qw androidboot.hardware=qcom /proc/cmdline && echo 'true' || echo 'false'`
  is64bit=`$bb grep -wq ro.product.cpu.abi=arm64-v8a $sys/build.prop && echo 'true' || echo 'false'`
  lib=`$bb find $sys/lib*|$bb grep -E "\/(libbluetooth|bluetooth\.default)\.so$"|$bb tail -n 1`

  # default(mostly arm64 exynos)=1   arm=2   qcom=3
  var=`if ( ! $is64bit || [[ $sdk -le 25 ]] ); then echo 2; elif $qcom; then echo 3; else echo 1; fi`

  hex=( \
  # default
  [131]=........f9031f2af3031f2a41000014 [1131]=1f2003d5f9031f2af3031f2a48000014 \
  [130]=........f3031f2af4031f2a3e000014 [1130]=1f2003d5f3031f2af4031f2a3e000014 \
  [129]=........f4031f2af3031f2ae8030032 [1129]=1f2003d5f4031f2af3031f2ae8031f2a \
  [128]=88000034e803003248070035 [1128]=1f2003d5e8031f2a48070035 \
  [127]=88000034e803003228050035 [1127]=1f2003d5e8031f2a28050035 \
  [126]=88000034e803003228050035 [1126]=1f2003d5e8031f2a28050035 \
  # arm
  [230]=18b14ff0000b00254ae0 [1230]=00204ff0000b002554e0 \
  [229]=..b100250120 [1229]=00bf00250020 \
  [228]=..b101200028 [1228]=00bf00200028 \
  [227]=09b1012032e0 [1227]=00bf002032e0 \
  [226]=08b1012031e0 [1226]=00bf002031e0 \
  [225]=087850bbb548 [1225]=08785ae1b548 \
  [224]=007840bb6a48 [1224]=0078c4e06a48 \
  # qcom
  [330]=88000054691180522925c81a69000037........ [1330]=04000014691180522925c81a69000037e0031f2a \
  [329]=88000054691180522925c81a69000037........ [1329]=04000014691180522925c81a69000037e0031f2a \
  [328]=7f1d0071e91700f9e83c0054 [1328]=e0031f2ae91700f9e8010014 \
  # what is happening samsung
  [429]=....0034f3031f2af4031f2a....0014 [1429]=1f2003d5f3031f2af4031f2a47000014 \
  # what again (T510)
  [529]=44387810b1002400254ae0 [1529]=44387800200024002556e0 \
  [530]=18b100244ff0000b4de0 [1530]=002000244ff0000b57e0 \
  # ... (T595)
  [629]=90387810b1002400254ae0 [1629]=90387800200024002558e0 )

  android=( [31]=12 [30]=11 [29]=10 [28]=Pie [27]='Oreo 8.1' [26]='Oreo 8.0' [25]='Nougat 7.1' [24]='Nougat 7.0' )
  model=SM-${bootloader:0:$((${#bootloader}-8))}

  ui_print "- $model on Android ${android[$sdk]} detected"

  ui_print "- Searching for relevant hex byte sequence"

  if [[ -f $lib.bak ]] ; then
    lib=$lib.bak
  else
    lib=$lib
  fi
  if [[ $var == 3 ]] && ! `$bb xxd -p $lib|$bb tr -d '\n'|$bb grep -qm1 ${hex[$var$sdk]}` ; then
    if `$bb xxd -p $lib|$bb tr -d '\n'|$bb grep -qm1 ${hex[1$var$sdk]}` ; then
      hex[$var$sdk]=already
    else
      var=1
    fi
  fi
  if ( [[ $var == 1 ]] && [[ $sdk -ge 29 ]] ) || ( [[ $var == 2 ]] && [[ $sdk -ge 28 ]] && [[ $sdk -le 29 ]] ) || ( [[ $var == 3 ]] && [[ $sdk -ge 29 ]] ) ; then
    hex[$var$sdk]=`$bb xxd -p $lib|$bb tr -d '\n'|$bb grep -om1 ${hex[$var$sdk]}`
  fi
  if [[ -z `$bb xxd -p $lib|$bb tr -d '\n'|$bb grep -om1 ${hex[$var$sdk]}` ]] ; then
    if `$bb xxd -p $lib|$bb tr -d '\n'|$bb grep -qm1 ${hex[1$var$sdk]}` ; then
      hex[$var$sdk]=already
    elif [[ $var == 1 ]] && [[ $sdk == 29 ]] && hex[4$sdk]=`$bb xxd -p $lib|$bb tr -d '\n'|$bb grep -om1 ${hex[4$sdk]}` && [[ ! -z ${hex[4$sdk]} ]] ; then
      var=4
    elif [[ $var == 2 ]] && [[ $sdk -ge 29 ]] ; then
      if `$bb xxd -p $lib|$bb tr -d '\n'|$bb grep -qm1 ${hex[5$sdk]}` ; then
        var=5
      elif `$bb xxd -p $lib|$bb tr -d '\n'|$bb grep -qm1 ${hex[6$sdk]}` ; then
        var=6
      fi
    fi
  fi
  echo -e "sdk=$sdk\nbootloader=$bootloader\nlib=$lib\nvar=$var\npre_hex=${hex[$var$sdk]}\npost_hex=${hex[1$var$sdk]}" > /tmp/bluetoothlibpatcher/tar/debug
}

patchvintfmanifest() {
  if `$bb grep -qr 'security.wsm' /vendor/etc/vintf` ; then
    ui_print "- Applying gear watch fix"
    # vendor resizing script based on afaneh92 work
    if ! `$bb dd if=/dev/zero of=/vendor/test bs=1 count=1 2>/dev/null` ; then
      ui_print "- Resizing fullfilled up vendor partition"
      dm_block=`$bb df -P|$bb grep /vendor|$bb awk '{print $1}'`
      old_blocks=`$bb df -P|$bb grep /vendor|$bb awk '{print $2}'`
      new_blocks=$(($old_blocks + 102400))
      $bb umount /vendor
      e2fsck -f $dm_block
      resize2fs $dm_block $new_blocksK
      e2fsck -E unshare_blocks $dm_block
      $bb mount -w /vendor
    fi
    $bb rm -f /vendor/test
    for i in `$bb grep -lr 'security.wsm' /vendor/etc/vintf`; do [[ ! -z $i ]] && $bb sed -i "$((`$bb awk '/security.wsm/ {print FNR}' $i`-1)),/<\/hal>/d" $i; done
  else
    ui_print "- Gear watch fix not needed/already applied"
  fi
}

patchlib() {
  ui_print "- Applying patch"
  if [[ ! -f $lib ]] ; then
    ui_print "- Library not found!"
    abort
  elif [[ ${hex[$var$sdk]} == already ]] ; then
    ui_print "- Library already patched!"
    exit_patch
  fi
  if [[ ! -z ${hex[$var$sdk]} ]] ; then
    if [[ $lib == *.bak ]] ; then
      ui_print "- Restoring the backup"
      $bb cp -acf $lib.bak $lib
    else
      ui_print "- Backuping library"
      $bb cp -acf $lib $lib.bak
    fi
  fi
  if [[ ! -z ${hex[$var$sdk]} ]] && `/tmp/bluetoothlibpatcher/bin/magiskboot hexpatch $lib ${hex[$var$sdk]} ${hex[1$var$sdk]} 2>/dev/null` ; then
    ui_print "- Successfully patched!"
  else
    ui_print "- Library not supported!"
    $bb cp -f $lib /tmp/bluetoothlibpatcher/tar
    $bb tar c -f /sdcard/BluetoothLibPatcher-files.tar -C /tmp/bluetoothlibpatcher/tar `$bb ls /tmp/bluetoothlibpatcher/tar`
    ui_print " "
    ui_print "- To get support please upload the file"
    ui_print "  BluetoothLibPatcher-files.tar created"
    ui_print "  in your internal storage to the XDA thread."
    ui_print " "
    abort
  fi
}

check
search
patchvintfmanifest
patchlib
clean
